use crate::util::model::member::{Member, MembersMigration};
use dioxus::prelude::{dioxus_elements::FileEngine, *};
use std::sync::Arc;

#[derive(Debug)]
struct UploadedFile {
    name: String,
    contents: String,
}

#[derive(PartialEq)]
enum Steps {
    Upload,
    Verify,
    Done,
}

#[component]
pub fn Migration() -> Element {
    let step = use_signal(|| Steps::Upload);
    let members_migration = use_signal(|| MembersMigration::new());

    rsx! {
        div {
            class: "flex flex-col items-center justify-center py-10",
            ul {
                class: "steps pb-10",
                li { class: "step step-primary", "Uploaden" },
                li {
                    class: "step",
                    class: if *step.read() == Steps::Verify { "step-primary" },
                    "Controleren"
                },
                li { class: "step", "Klaar" },
            },
            match *step.read() {
                Steps::Upload => rsx! { Upload { step: step, members_migration: members_migration } },
                Steps::Verify => rsx! { Verify { members_migration: members_migration} },
                Steps::Done => rsx! { Verify { members_migration: members_migration } },
            }
        }
    }
}

#[component]
fn Upload(step: Signal<Steps>, members_migration: Signal<MembersMigration>) -> Element {
    let mut file_uploaded = use_signal(|| None);

    let mut loading = use_signal(|| false);

    let read_files = move |file_engine: Arc<dyn FileEngine>| async move {
        let files = file_engine.files();
        for file_name in &files {
            if let Some(contents) = file_engine.read_file_to_string(file_name).await {
                file_uploaded.set(Some(UploadedFile {
                    name: file_name.clone(),
                    contents,
                }));
            }
        }
    };

    let upload_files = move |evt: FormEvent| async move {
        if let Some(file_engine) = evt.files() {
            read_files(file_engine).await;
        }
    };

    let sumbit = move |_evt: FormEvent| async move {
        match &*file_uploaded.read() {
            Some(file) => {
                loading.set(true);

                if let Ok(response) = upload_members_list(file.contents.clone()).await {
                    members_migration.set(response);
                    step.set(Steps::Verify);
                    loading.set(false);
                }
                loading.set(false);
            }
            None => tracing::info!("File doesn't exists"),
        }
    };

    rsx! {
        form {
            class: "flex flex-col items-center w-full h-full max-w-md mx-auto px-2",
            onsubmit: sumbit,
            h2 { class: "text-xl mb-5", "Selecteer de ledenlijst" },
            input {
                r#type: "file",
                class: "file-input file-input-bordered w-full",
                accept: ".csv",
                multiple: false,
                autocomplete: "off",
                onchange: upload_files,
            }
            button {
                class: "btn btn-primary btn-wide mt-5",
                disabled: file_uploaded.read().is_none() || loading(),
                if loading() {
                    span { class: "loading loading-spinner" }
                }
                "Uploaden"
            }
        }
    }
}

#[component]
fn Verify(members_migration: Signal<MembersMigration>) -> Element {
    let mut loading = use_signal(|| false);

    rsx! {
        div {
            class: "flex flex-col items-center justify-center w-full mx-auto px-2",
            h2 { class: "text-xl mb-5", "Controleer de verandering" },
            div {
                class: "flex flex-wrap gap-5",
                div {
                    class: "card bg-base-200 p-5",
                    h2 { class: "card-title mb-1", "Toevoegen" }
                    MembersTable { members: members_migration.read().inserted.clone() }
                }
                div {
                    class: "card bg-base-200 p-5",
                    h2 { class: "card-title mb-1", "Verwijderen" }
                    MembersTable { members: members_migration.read().removed.clone() }
                }
                div {
                    class: "card bg-base-200 p-5",
                    h2 { class: "card-title mb-1", "Updaten" }
                    MembersTable { members: members_migration.read().updated.clone() }
                }
            }
            button {
                class: "btn btn-primary btn-wide mt-5",
                disabled: loading(),
                if loading() {
                    span { class: "loading loading-spinner" }
                }
                "Toepassen"
            }
        }
    }
}

#[component]
fn MembersTable(members: Vec<Member>) -> Element {
    rsx! {
        div {
            class: "h-[30rem] w-96 overflow-auto font-normal",
            table {
                class: "table table-pin-rows",
                thead {
                    tr {
                        th { "Relatiecode" }
                        th { "Naam" }
                    }
                }
                tbody {
                    for member in members {
                        tr {
                            th { "{member.id}" }
                            th { "{member.name.full}" }
                        }
                    }
                }
            }

        }
    }
}

#[server]
async fn upload_members_list(input: String) -> Result<MembersMigration, ServerFnError> {
    match Member::migrate_proposal(input).await {
        Ok(r) => Ok(r),
        Err(err) => Err(ServerFnError::new(err.to_string())),
    }
}

#[server]
async fn check_members_list(correct: bool, id: u32) -> Result<(), ServerFnError> {
    tracing::info!("boe");
    Ok(())
}
